//version: 1.0
#region Using directives

using System;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;

#endregion

namespace WindowMessages

{
    public class WindowMessage : IDisposable
    {
        public static double version = 1.0;

        private List<uint> TrappedWinMsgs;
        private IntPtr OldWindowProc;
        private delegate int _WindowProc(IntPtr hwnd, uint msg, uint wParam, int lParam);
        private _WindowProc NewWinProcDel;

        private Control ictl;
        private IntPtr ihwnd;
        private uint iwmsg;
        private int ilparam;
        private uint iwparam;
        private bool ihandled;
        private bool ihooked;
        private bool itrapall;

        private object[] eventObject;
        public event EventHandler TrapEvent;


        // constructor
        public WindowMessage(Control ctl)
        {
            ictl = ctl;
            ihwnd = ctl.Handle;
            ihandled = false;
            ihooked = false;
            eventObject = new object[] { this, "TrapEvent" };
            NewWinProcDel = new _WindowProc(NewWindowProc);
            TrappedWinMsgs = new List<uint>(16);
        }


        // event handlers
        private int WmTrapped(IntPtr hwnd, uint wmsg, uint wParam, int lParam, ref bool handled)
        {
            if (TrapEvent != null)
            {
                ihwnd = hwnd;
                iwmsg = wmsg;
                iwparam = wParam;
                ilparam = lParam;
                TrapEvent(eventObject, null);
            }
            handled = ihandled;
            return 1;
        }

        private int NewWindowProc(IntPtr hwnd, uint wmsg, uint wParam, int lParam)
        {
            int idx = TrappedWinMsgs.BinarySearch(wmsg);
            if (itrapall ||  idx >= 0)
            {
                bool handled = false;
                int num = WmTrapped(hwnd, wmsg, wParam, lParam, ref handled);
                if (handled)
                {
                    return num;
                }
            }
            return CallWindowProc(OldWindowProc, hwnd, wmsg, wParam, lParam);
        }

        // public methods
        public IntPtr GetHandle(Control control)
        {
            return control.Handle;
        }

        public bool IsTrapped(uint wmsg)
        {
            if (TrappedWinMsgs.BinarySearch(wmsg) >= 0)
                return true;
            return false;
        }

        public int PostMessageInt32(IntPtr handle, uint wmsg, uint wParam, int lParam)
        {
            return PostMessage(handle, wmsg, wParam, lParam);
        }

        public int PostMessageString(IntPtr handle, uint wmsg, uint wParam, string lParam)
        {
            return PostMessage(handle, wmsg, wParam, lParam);
        }

        public int SendMessageInt32(IntPtr handle, uint wmsg, uint wParam, int lParam)
        {
            return SendMessage(handle, wmsg, wParam, lParam);
        }

        public int SendMessageString(IntPtr handle, uint wmsg, uint wParam, string lParam)
        {
            return SendMessage(handle, wmsg, wParam, lParam);
        }

        public void Start()
        {
            if (!ihooked)
            {
                OldWindowProc = SetWindowLong(ihwnd, GWL_WNDPROC, Marshal.GetFunctionPointerForDelegate(NewWinProcDel));
                ihooked = true;
            }
        }

        public void Stop()
        {
            if (ihooked)
            {
                SetWindowLong(ihwnd, GWL_WNDPROC, OldWindowProc);
                ihooked = false;
            }
        }

        public void Trap(uint wmsg)
        {
            if (TrappedWinMsgs.BinarySearch(wmsg) < 0)
            {
                TrappedWinMsgs.Add(wmsg);
                TrappedWinMsgs.Sort();
            }

        }

        public void TrapAll()
        {
            itrapall = true;
        }

        public void TrapRange(uint wmsg1, uint wmsg2)
        {
            if (wmsg2 < wmsg1)
            {
                uint temp = wmsg1;
                wmsg1 = wmsg2;
                wmsg2 = temp;
            }
            for (uint i = wmsg1; i <= wmsg2; i++)
                Trap(i);           
        }

        public void Untrap(uint wmsg)
        {
            int idx = TrappedWinMsgs.BinarySearch(wmsg);
            if (idx >= 0)
            {
                TrappedWinMsgs.RemoveAt(idx);
            }
        }

        public void UntrapRange(uint wmsg1, uint wmsg2)
        {
            if (wmsg2 < wmsg1)
            {
                uint temp = wmsg1;
                wmsg1 = wmsg2;
                wmsg2 = temp;
            }
            for (uint i = wmsg1; i <= wmsg2; i++)
                Untrap(i);
        }

        public void UntrapAll()
        {
            TrappedWinMsgs.Clear();
            itrapall = false;
        }


        // properties
        public double DllVersion
        { get { return version; } }

        public IntPtr Handle
        { get { return ihwnd; } }

        public int lParam
        { get { return ilparam; } }

        public bool Started
        { get { return ihooked; } }

        public uint wMsg
        { get { return iwmsg; } }

        public bool Handled
        { get{  return ihandled; } set{ ihandled = value; } }

        public uint wParam
        { get { return iwparam; } }

        // dispose

        public void Dispose()
        {
            Stop();
        }


        // Native code stuff ----------------------------------------------------------------

        private const int GWL_WNDPROC = -4;

        //[DllImport("coredll.dll")]
        [DllImport("user32.dll")]
        private static extern int CallWindowProc(IntPtr lpPrevWndFunc, IntPtr hwnd, uint msg, uint wParam, int lParam);

        //[DllImport("coredll.dll")]
        [DllImport("user32.dll")]
        private static extern int PostMessage(IntPtr hwnd, uint wMsg, uint wParam, int lParam);

        //[DllImport("coredll.dll")]
        [DllImport("user32.dll")]
        private static extern int PostMessage(IntPtr hwnd, uint wMsg, uint wParam, string lParam);

        //[DllImport("coredll.dll")]
        [DllImport("user32.dll")]
        private static extern int SendMessage(IntPtr hwnd, uint wMsg, uint wParam, int lParam);

        //[DllImport("coredll.dll")]
        [DllImport("user32.dll")]
        private static extern int SendMessage(IntPtr hwnd, uint wMsg, uint wParam, string lParam);

        //[DllImport("coredll.dll")]
        [DllImport("user32.dll")]
        private static extern IntPtr SetWindowLong(IntPtr hwnd, int nIndex, IntPtr dwNewLong);

    }


}
